package reborncore.common.util;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.inventory.InventoryLargeChest;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.tileentity.TileEntityChest;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.ITextComponent;
import net.minecraft.world.World;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


//From OpenBlocksLib: https://github.com/OpenMods/OpenModsLib
public class InventoryHelper {

    public static void tryInsertStack(IInventory targetInventory, int slot, ItemStack stack, boolean canMerge) {
        if (targetInventory.func_94041_b(slot, stack)) {
            ItemStack targetStack = targetInventory.func_70301_a(slot);
            if (targetStack == null) {
                int space = targetInventory.func_70297_j_();
                int mergeAmount = Math.min(space, stack.field_77994_a);

                ItemStack copy = stack.func_77946_l();
                copy.field_77994_a = mergeAmount;
                targetInventory.func_70299_a(slot, copy);
                stack.field_77994_a -= mergeAmount;
            } else if (canMerge) {
                if (targetInventory.func_94041_b(slot, stack) && areMergeCandidates(stack, targetStack)) {
                    int space = Math.min(targetInventory.func_70297_j_(), targetStack.func_77976_d()) - targetStack.field_77994_a;
                    int mergeAmount = Math.min(space, stack.field_77994_a);

                    ItemStack copy = targetStack.func_77946_l();
                    copy.field_77994_a += mergeAmount;
                    targetInventory.func_70299_a(slot, copy);
                    stack.field_77994_a -= mergeAmount;
                }
            }
        }
    }

    protected static boolean areMergeCandidates(ItemStack source, ItemStack target) {
        return source.func_77969_a(target) && ItemStack.func_77970_a(source, target) && target.field_77994_a < target.func_77976_d();
    }

    public static void insertItemIntoInventory(IInventory inventory, ItemStack stack) {
        insertItemIntoInventory(inventory, stack, null, -1);
    }

    public static void insertItemIntoInventory(IInventory inventory, ItemStack stack, EnumFacing side, int intoSlot) {
        insertItemIntoInventory(inventory, stack, side, intoSlot, true);
    }

    public static void insertItemIntoInventory(IInventory inventory, ItemStack stack, EnumFacing side, int intoSlot, boolean doMove) {
        insertItemIntoInventory(inventory, stack, side, intoSlot, doMove, true);
    }

    public static void insertItemIntoInventory(IInventory inventory, ItemStack stack, EnumFacing side, int intoSlot, boolean doMove, boolean canStack) {
        if (stack == null) return;

        IInventory targetInventory = inventory;

        if (!doMove) {
            targetInventory = new GenericInventory("temporary.inventory", false, targetInventory.func_70302_i_());
            ((GenericInventory) targetInventory).copyFrom(inventory);
        }

        int i = 0;
        int[] attemptSlots = new int[0];

        if (inventory instanceof ISidedInventory && side != null) {
            attemptSlots = ((ISidedInventory) inventory).func_180463_a(side);
            if (attemptSlots == null)
                attemptSlots = new int[0];
        } else {
            attemptSlots = new int[inventory.func_70302_i_()];
            for (int a = 0; a < inventory.func_70302_i_(); a++)
                attemptSlots[a] = a;
        }
        if (intoSlot > -1) {
            Set<Integer> x = new HashSet<Integer>();
            for (int attemptedSlot : attemptSlots)
                x.add(attemptedSlot);

            if (x.contains(intoSlot))
                attemptSlots = new int[]{intoSlot};
            else attemptSlots = new int[0];
        }
        while (stack.field_77994_a > 0 && i < attemptSlots.length) {
            if (side != null && inventory instanceof ISidedInventory)
                if (!((ISidedInventory) inventory).func_180462_a(attemptSlots[i], stack, side.func_176734_d())) {
                    i++;
                    continue;
                }

            tryInsertStack(targetInventory, attemptSlots[i], stack, canStack);
            i++;
        }
    }

    public static int testInventoryInsertion(IInventory inventory, ItemStack item, EnumFacing side) {
        if (item == null || item.field_77994_a == 0)
            return 0;
        item = item.func_77946_l();

        if (inventory == null)
            return 0;

        inventory.func_70302_i_();

        int itemSizeCounter = item.field_77994_a;
        int[] availableSlots = new int[0];

        if (inventory instanceof ISidedInventory)
            availableSlots = ((ISidedInventory) inventory).func_180463_a(side);
        else {
            availableSlots = buildSlotsForLinearInventory(inventory);
        }

        for (int i : availableSlots) {
            if (itemSizeCounter <= 0)
                break;

            if (!inventory.func_94041_b(i, item))
                continue;

            if (side != null && inventory instanceof ISidedInventory)
                if (!((ISidedInventory) inventory).func_180462_a(i, item, side.func_176734_d()))
                    continue;

            ItemStack inventorySlot = inventory.func_70301_a(i);
            if (inventorySlot == null)
                itemSizeCounter -= Math.min(Math.min(itemSizeCounter, inventory.func_70297_j_()), item.func_77976_d());
            else if (areMergeCandidates(item, inventorySlot)) {
                int space = Math.min(inventory.func_70297_j_(), inventorySlot.func_77976_d()) - inventorySlot.field_77994_a;
                itemSizeCounter -= Math.min(itemSizeCounter, space);
            }
        }

        if (itemSizeCounter != item.field_77994_a) {
            itemSizeCounter = Math.max(itemSizeCounter, 0);
            return item.field_77994_a - itemSizeCounter;
        }

        return 0;
    }

    public static IInventory getInventory(World world, int x, int y, int z) {
        BlockPos pos = new BlockPos(x, y, z);
        TileEntity tileEntity = world.func_175625_s(pos);
        if (tileEntity instanceof TileEntityChest) {
            TileEntityChest chest = (TileEntityChest) tileEntity;

            TileEntityChest adjacent = null;

            if (chest.field_145991_k != null) {
                adjacent = chest.field_145991_k;
            }

            if (chest.field_145990_j != null) {
                adjacent = chest.field_145990_j;
            }

            if (chest.field_145992_i != null) {
                adjacent = chest.field_145992_i;
            }

            if (chest.field_145988_l != null) {
                adjacent = chest.field_145988_l;
            }

            if (adjacent != null) {
                return new InventoryLargeChest("", chest, adjacent);
            }
            return chest;
        }
        return tileEntity instanceof IInventory ? (IInventory) tileEntity : null;
    }

    public static IInventory getInventory(World world, int x, int y, int z, EnumFacing direction) {
        if (direction != null && direction != null) {
            x += direction.func_82601_c();
            y += direction.func_96559_d();
            z += direction.func_82599_e();
        }
        return getInventory(world, x, y, z);

    }

    public static IInventory getInventory(IInventory inventory) {
        if (inventory instanceof TileEntityChest) {
            TileEntity te = (TileEntity) inventory;
            return getInventory(te.func_145831_w(), te.func_174877_v().func_177958_n(), te.func_174877_v().func_177956_o(), te.func_174877_v().func_177952_p());
        }
        return inventory;
    }

    public static int[] buildSlotsForLinearInventory(IInventory inv) {
        int[] slots = new int[inv.func_70302_i_()];
        for (int i = 0; i < slots.length; i++)
            slots[i] = i;

        return slots;
    }

    public static class GenericInventory implements IInventory {

        protected String inventoryTitle;
        protected int slotsCount;
        protected ItemStack[] inventoryContents;
        protected boolean isInvNameLocalized;

        public GenericInventory(String name, boolean isInvNameLocalized, int size) {
            this.isInvNameLocalized = isInvNameLocalized;
            slotsCount = size;
            inventoryTitle = name;
            inventoryContents = new ItemStack[size];
        }

        @Override
        public ItemStack func_70298_a(int par1, int par2) {
            if (inventoryContents[par1] != null) {
                ItemStack itemstack;

                if (inventoryContents[par1].field_77994_a <= par2) {
                    itemstack = inventoryContents[par1];
                    inventoryContents[par1] = null;
                    return itemstack;
                }

                itemstack = inventoryContents[par1].func_77979_a(par2);
                if (inventoryContents[par1].field_77994_a == 0)
                    inventoryContents[par1] = null;

                return itemstack;
            }
            return null;
        }

        @Override
        public int func_70297_j_() {
            return 64;
        }

        @Override
        public int func_70302_i_() {
            return slotsCount;
        }

        @Override
        public ItemStack func_70301_a(int i) {
            return inventoryContents[i];
        }

        public ItemStack getStackInSlot(Enum<?> i) {
            return func_70301_a(i.ordinal());
        }

        @Override
        public ItemStack func_70304_b(int i) {
            if (i >= inventoryContents.length)
                return null;

            if (inventoryContents[i] != null) {
                ItemStack itemstack = inventoryContents[i];
                inventoryContents[i] = null;
                return itemstack;
            }

            return null;
        }

        public boolean isItem(int slot, Item item) {
            return inventoryContents[slot] != null && inventoryContents[slot].func_77973_b() == item;
        }

        @Override
        public boolean func_94041_b(int i, ItemStack itemstack) {
            return true;
        }

        @Override
        public int func_174887_a_(int id) {
            return 0;
        }

        @Override
        public void func_174885_b(int id, int value) {

        }

        @Override
        public int func_174890_g() {
            return 0;
        }

        @Override
        public void func_174888_l() {

        }

        @Override
        public boolean func_70300_a(EntityPlayer entityplayer) {
            return true;
        }

        @Override
        public void func_174889_b(EntityPlayer player) {

        }

        @Override
        public void func_174886_c(EntityPlayer player) {

        }

        public void clearAndSetSlotCount(int amount) {
            slotsCount = amount;
            inventoryContents = new ItemStack[amount];
        }

        public void readFromNBT(NBTTagCompound tag) {
            if (tag.func_74764_b("size"))
                slotsCount = tag.func_74762_e("size");

            NBTTagList nbttaglist = tag.func_150295_c("Items", 10);
            inventoryContents = new ItemStack[slotsCount];
            for (int i = 0; i < nbttaglist.func_74745_c(); i++) {
                NBTTagCompound stacktag = nbttaglist.func_150305_b(i);
                int j = stacktag.func_74771_c("Slot");
                if (j >= 0 && j < inventoryContents.length)
                    inventoryContents[j] = ItemStack.func_77949_a(stacktag);
            }
        }

        @Override
        public void func_70299_a(int i, ItemStack itemstack) {
            inventoryContents[i] = itemstack;

            if (itemstack != null && itemstack.field_77994_a > func_70297_j_())
                itemstack.field_77994_a = func_70297_j_();
        }

        public void writeToNBT(NBTTagCompound tag) {
            tag.func_74768_a("size", func_70302_i_());
            NBTTagList nbttaglist = new NBTTagList();
            for (int i = 0; i < inventoryContents.length; i++) {
                if (inventoryContents[i] != null) {
                    NBTTagCompound stacktag = new NBTTagCompound();
                    stacktag.func_74774_a("Slot", (byte) i);
                    inventoryContents[i].func_77955_b(stacktag);
                    nbttaglist.func_74742_a(stacktag);
                }
            }
            tag.func_74782_a("Items", nbttaglist);
        }

        public void copyFrom(IInventory inventory) {
            for (int i = 0; i < inventory.func_70302_i_(); i++)
                if (i < func_70302_i_()) {
                    ItemStack stack = inventory.func_70301_a(i);
                    if (stack != null)
                        func_70299_a(i, stack.func_77946_l());
                    else func_70299_a(i, null);
                }
        }

        public List<ItemStack> contents() {
            return Arrays.asList(inventoryContents);
        }


        @Override
        public void func_70296_d() {
        }

        @Override
        public String func_70005_c_() {
            return null;
        }

        @Override
        public boolean func_145818_k_() {
            return false;
        }

        @Override
        public ITextComponent func_145748_c_() {
            return null;
        }
    }
}
